//*****************************************************************************
//  Copyright (c) 2021 Glenfly Tech Co., Ltd.. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Glenfly Tech Co., Ltd..;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Glenfly Tech Co., Ltd..
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************

#ifndef  __H8_VIDEO_H__
#define  __H8_VIDEO_H__

#include "cam_mgr.h"

//#define H8V_VERSION_CODE   KERNEL_VERSION(4,11,0)
#ifndef VIP_V4L2_VERSION_CODE
#define VIP_V4L2_VERSION_CODE LINUX_VERSION_CODE
#endif

#define MAX_WIDTH           4160
#define MAX_HEIGHT          3120
#define MAX_FIFO_ITEM_NUM   16

#define MAX_FORMAT_CAPS     3

typedef enum _H8V_CALLBACK_TYPE{
    H8V_CALLBACK_START = 0,
    H8V_CALLBACK_VIDEO,
    H8V_CALLBACK_PICTURE,
    H8V_CALLBACK_END,
}H8V_CALLBACK_TYPE;

typedef struct _FIFOItem
{
    unsigned int addr;
    unsigned int pitch;
}FIFOItem;

typedef struct _FIFOInstance
{
    FIFOItem     item[MAX_FIFO_ITEM_NUM];
    unsigned int start_pos;
    unsigned int num;
}FIFOInstance;

struct h8v_fmt
{
    char  *name;
    u32   fourcc;          /* v4l2 format id */
    int   depth;
};

/* buffer for one video frame */
struct h8v_buffer
{
    /* common v4l buffer stuff -- must be first */
    struct vb2_buffer	vb;
    struct cam_buf      vip_buf;
    struct list_head	list;
    struct h8v_fmt     *fmt;

};

struct h8v_dmaqueue
{
    struct list_head    active;

    struct task_struct *kthread;
    int                 frame;
    int                 ini_jiffies;
};

struct h8_video_dev
{
    /* V4L2 */
    struct v4l2_device 	v4l2_dev;
    struct video_device *video_dev;
    int                 id;
    int                 video_index;
    struct platform_device      *pdev;
    struct cam_mgr              *mgr;
    struct cam_caps     caps;

    struct v4l2_ctrl_handler    ctrl_handler;

    spinlock_t          slock;
    struct mutex        mutex;

    struct vb2_queue    vb_vidq;
    struct mutex        queue_lock;

    unsigned int        width;    //currently using size
    unsigned int        height;
    struct h8v_fmt     *fmt;
    struct v4l2_fract   fmt_fps;

    enum v4l2_field     field;
    unsigned int        field_count;

    unsigned int        input;
    unsigned int        opened;

    unsigned int        frame_counter;

    // -----------------------------------
    bool                skip_ctrl_hw;

    struct v4l2_ctrl    *tonemap_curve;
    unsigned int        tonemap_curve_data[295];            // maybe 295*3?


    struct v4l2_ctrl    *stream_init;
    Stream_Init         stream_init_data;
    struct v4l2_ctrl    *main_ctrl;
    ISP_Ctrl            main_ctrl_data;
    
    // ---- volatile ----
    volatile bool       stream_stoped;
};

//==============================================
typedef int (*h8v_ctrl_t)(struct h8_video_dev* dev);

//=============== import func ==================


#endif  //__H8_VIDEO_H__
