//*****************************************************************************
//  Copyright (c) 2021 Glenfly Tech Co., Ltd.. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Glenfly Tech Co., Ltd..;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Glenfly Tech Co., Ltd..
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************

#ifndef __gf_def_h__
#define __gf_def_h__

#define MAX_SCREENS         4

//Fix-me:
// need set MAX_CORE_CRTCS as 3 in CHX001, just keep it as 2 in e2uma validation.
// currently cannot set 3, because there is no vsync interrupt for the 3rd IGA, this may lead KMD pending in some cases
//
#define MAX_CORE_CRTCS      4

#define OUTPUT_NAME_LEN     16

#define MAX_ENGINE_COUNT   10
#define ALL_ENGINE_MASK    ((1 << MAX_ENGINE_COUNT) - 1)

typedef union
{
#ifndef __BIG_ENDIAN__
 struct
 {
    unsigned int low32;
    unsigned int  high32;
 };
 #else 
 struct
 {
    unsigned int  high32;
    unsigned int  low32;
 };
 #endif
    unsigned long long quad64;
}large_inter;

#define ALIGN8 __attribute__ ((aligned(8)))

#ifndef UNUSED
#define UNUSED(x) (void)(x)
#endif

typedef unsigned long long gf_ptr64_t;

typedef struct gf_clip_rect
{
     short x1;
     short y1;
     short x2;
     short y2;
}gf_clip_rect_t;

typedef enum gf_miu_reg_type
{
    GF_MIU_MMIO = 1,
    GF_MIU_PMC = 2,
    GF_MIU_PMU = 3,
}gf_miu_reg_type_t;

typedef  struct
{
    struct  //get from adapter
    {
        unsigned int  family_id;
        unsigned int  generic_id;
        unsigned int  chip_id;
        unsigned char*   mmio;
        unsigned int  mmio_size;
        unsigned int  primary;
        unsigned int  fb_bus_addr;
        unsigned int  fb_total_size;
        unsigned char *gpio;
        unsigned int  patch_fence_intr_lost:1;
        unsigned int  init_render:1;
        union
        {
            unsigned int  adp_flags;
            struct
            {
                unsigned int  run_on_qt:1;
                unsigned int  Reserved:31;
            };
        };
    };
    struct   //set to adapter
    {
        unsigned int low_top_addr;
        unsigned int snoop_only;
        unsigned int chan_num;
        unsigned int avai_mem_size_mb; //in Mega Bytes
        unsigned int total_mem_size_mb; //in Mega Bytes
        unsigned int chip_slice_mask;
    };
}adapter_info_t;

typedef enum
{
    GF_STREAM_PS = 0,
    GF_STREAM_SS,
    GF_STREAM_TS,
    GF_STREAM_4S,
    GF_STREAM_5S,
    GF_STREAM_6S,
    GF_STREAM_MAX,
}GF_STREAM_TYPE;

typedef enum
{
    GF_PLANE_PS = 0,
    GF_PLANE_SS,
    GF_PLANE_TS,
    GF_PLANE_FS,
    GF_MAX_PLANE,
}GF_PLANE_TYPE;


typedef enum
{
    GF_AUDIO_FORMAT_REFER_TO_STREAM_HEADER,
    GF_AUDIO_FORMAT_LPCM,
    GF_AUDIO_FORMAT_AC_3,
    GF_AUDIO_FORMAT_MPEG_1,
    GF_AUDIO_FORMAT_MP3,
    GF_AUDIO_FORMAT_MPEG_2,
    GF_AUDIO_FORMAT_AAC_LC,
    GF_AUDIO_FORMAT_DTS,
    GF_AUDIO_FORMAT_ATRAC,
    GF_AUDIO_FORMAT_DSD,
    GF_AUDIO_FORMAT_E_AC_3,
    GF_AUDIO_FORMAT_DTS_HD,
    GF_AUDIO_FORMAT_MLP,
    GF_AUDIO_FORMAT_DST,
    GF_AUDIO_FORMAT_WMA_PRO,
    GF_AUDIO_FORMAT_HE_AAC,
    GF_AUDIO_FORMAT_HE_AAC_V2,
    GF_AUDIO_FORMAT_MPEG_SURROUND
}GF_HDMI_AUDIO_FORMAT_TYPE;

typedef struct
{
    GF_HDMI_AUDIO_FORMAT_TYPE   format;
    unsigned int                 max_channel_num;
    union
    {
        struct
        {
            unsigned int         SR_32kHz             :1; /* Bit0 = 1, support sample rate of 32kHz */
            unsigned int         SR_44_1kHz           :1; /* Bit1 = 1, support sample rate of 44.1kHz */
            unsigned int         SR_48kHz             :1; /* Bit2 = 1, support sample rate of 48kHz */
            unsigned int         SR_88_2kHz           :1; /* Bit3 = 1, support sample rate of 88.2kHz */
            unsigned int         SR_96kHz             :1; /* Bit4 = 1, support sample rate of 96kHz */
            unsigned int         SR_176_4kHz          :1; /* Bit5 = 1, support sample rate of 176.4kHz */
            unsigned int         SR_192kHz            :1; /* Bit6 = 1, support sample rate of 192kHz */
            unsigned int         reserved             :25;
        }sample_rate;

        unsigned int             sample_rate_unit;
    };

    union
    {
        unsigned int             unit;

        // for audio format: LPCM
        struct
        {
            unsigned int         BD_16bit             :1; /* Bit0 = 1, support bit depth of 16 bits */
            unsigned int         BD_20bit             :1; /* Bit1 = 1, support bit depth of 20 bits */
            unsigned int         BD_24bit             :1; /* Bit2 = 1, support bit depth of 24 bits */
            unsigned int         reserved             :29;
        }bit_depth;

        // for audio format: AC-3, MPEG-1, MP3, MPED-2, AAC LC, DTS, ATRAC
        unsigned int             max_bit_Rate; // unit: kHz

        // for audio format: DSD, E-AC-3, DTS-HD, MLP, DST
        unsigned int             audio_format_depend_value; /* for these audio formats, this value is defined in 
                                                            it's corresponding format-specific documents*/

        // for audio format: WMA Pro
        struct
        {
            unsigned int         value                :3;
            unsigned int         reserved             :29;
        }profile;
    };
}gf_hdmi_audio_format_t;

typedef struct
{
    unsigned int            num_formats;
    gf_hdmi_audio_format_t audio_formats[16];
}gf_hdmi_audio_formats;

typedef struct
{
    int crtc_index;
    int *hpos;
    int *vpos;
    int *vblk;
    int *in_vblk;
}gf_get_counter_t;

typedef struct
{
    unsigned int engine_mask;  /* in */
    unsigned int  hAllocation;  /* in */
}gf_wait_allocation_idle_t;

typedef struct
{
    int input_size;    /* in */
    int output_size;   /* in */

    gf_ptr64_t input  ALIGN8;/* in */
    gf_ptr64_t output ALIGN8;/* output */
}gf_escape_call_t;

typedef struct
{
    int type;
    union {
        unsigned int ivalue;
        unsigned long long lvalue;
    };
}gf_video_cmd_t;

typedef enum  
{
    GF_QUERY_VBIOS_VERSION,
    GF_QUERY_TOTAL_VRAM_SIZE,
    GF_QUERY_CPU_VISIBLE_VRAM_SIZE,
    GF_QUERY_RESERV_VRAM_SIZE,
    GF_QUERY_CHIP_ID,
    GF_QUERY_CRTC_OUTPUT,
    GF_QUERY_VSYNC_CNT,
    GF_QUERY_HEIGHT_ALIGN,
    GF_QUERY_LOCAL_VRAM_TYPE,
    GF_QUERY_ENGINE_CLOCK,
    GF_QUERY_VCLK,
    GF_QUERY_MCLK,
    GF_QUERY_VMEM_CLOCK,
    GF_QUERY_VRAM_USED,
    GF_QUERY_GART_USED,
    GF_QUERY_REGISTER_U32,
    GF_QUERY_REGISTER_U64,
    GF_QUERY_INFO_MAX,
    GF_QUERY_I_CLOCK,
    GF_QUERY_CPU_FREQUENCE,
    GF_SET_I_CLOCK,
    GF_QUERY_PMU_REGISTER_U32,
    GF_SET_CPU_FREQUENCE,
    GF_QUERY_VENDOR_ID,
    GF_QUERY_DEVICE_ID,
    GF_QUERY_REVISION_ID,
    GF_QUERY_SEGMENT_FREE_SIZE,
    GF_QUERY_HW_HANG,
    GF_QUERY_SECURED_ON,
    GF_QUERY_PAGE_SWIZZLE_SUPPORT,
    GF_QUERY_PENDING_FRAME_NUM,
    GF_QUERY_PMC_REGISTER_U32,
    GF_QUERY_ALLOCATION_INFO,
    GF_QUERY_GPU_TEMPERATURE,
    GF_QUERY_GPU_TIME_STAMP,
    GF_QUERY_LOCAL_ALLOCATION_MAX_SIZE,
    GF_QUERY_VCP_INDEX,
    GF_QUERY_RET_VCP_INDEX,
    GF_QUERY_VCP_INFO,
    GF_QUERY_VIDEO_SEQ_INDEX,
    GF_QUERY_RET_VIDEO_SEQ_INDEX,
    GF_QUERY_ADAPTER_INFO,

    GF_QUERY_PCIE_LINK_WIDTH,
    GF_QUERY_ACTIVE_ENGINE_COUNT,

    GF_QUERY_GET_VIDEO_BRIDGE_BUFFER,
    GF_QUERY_CHIP_SLICE_MASK,
    GF_SET_MIU_REGISTER_U32,
    GF_QUERY_MIU_REGISTER_U32,
    GF_QUERY_DIAGS,

    GF_QUERY_ALLOCATION_INFO_KMD,
}GF_QUERY_INFO_TYPE;

typedef struct
{
    unsigned int type;    /* in */
    unsigned int argu;    /* in */
    union
    {
        int          signed_value;           /* out */
        unsigned int value;                  /* out */
        unsigned long long value64;          /* out */
        void         *buf;                   /* out */
    };
    struct{
        unsigned int device_id;
        unsigned int vendor_id;
        unsigned long long total_vram_size;
        unsigned long long cpu_visible_size;
        unsigned int pci_link_width;
        unsigned int pci_link_speed;
        unsigned int engine_clk;
        unsigned int memory_clk;
        unsigned int video_clk;
    }diags;
}gf_query_info_t;

typedef gf_query_info_t gf_config_info_t;

/* 
if umd need poll events,  the definition need consistent between interrupt.h and gf_def.h
*/

typedef struct gf_begin_perf_event_tag
{
    unsigned int max_event_num;     /* in */
    unsigned int attrib_list[64];   /* in */
} gf_begin_perf_event_t;

typedef struct gf_end_perf_event_tag
{
    unsigned int placeholder;       /* in */
} gf_end_perf_event_t;

typedef struct gf_get_perf_event_tag
{
    unsigned int max_event_num;             /* in */
    unsigned int max_event_buffer_size;     /* in */
    void *event_buffer;                     /* in/out */
    unsigned int event_filled_num;          /* out */
    unsigned int event_buffer_filled_size;  /* out */
    unsigned int event_lost_num;            /* out */
    unsigned long long event_lost_timestamp;/* out */

    void *isr_event_buffer;                 /* in/out */
    unsigned int isr_filled_num;            /* out */
    unsigned int isr_filled_size;           /* out */
} gf_get_perf_event_t;

typedef struct _gf_miu_list_item
{
    unsigned int write : 1;
    unsigned int value_valid : 1;
    unsigned int miu_type : 6;
    unsigned int miu_offset : 24;
    unsigned int miu_value;
}gf_miu_list_item_t;

typedef struct gf_begin_miu_dump_tag
{
    unsigned int max_event_num;
}gf_begin_miu_dump_perf_event_t;

typedef struct gf_end_miu_dump_tag
{
    unsigned int place_hoder;
}gf_end_miu_dump_perf_event_t;

typedef struct gf_get_miu_dump_tag
{
    unsigned int max_event_num;
    unsigned int max_event_buffer_size;
    void *event_buffer;
    unsigned int event_filled_num;
    unsigned int event_buffer_filled_size;
    unsigned int event_lost_num;
    unsigned long long event_lost_timestamp;
}gf_get_miu_dump_perf_event_t;

typedef struct gf_direct_get_miu_dump_tag
{
    gf_miu_list_item_t *miu_table;
    unsigned int        miu_table_length;
    unsigned int        timestamp_high;
    unsigned int        timestamp_low;
}gf_direct_get_miu_dump_perf_event_t;

typedef struct gf_miu_reg_list_tag
{
    unsigned int          miu_table_length;
    gf_miu_list_item_t   *miu_table;
}gf_miu_reg_list_perf_event_t;

typedef struct gf_perf_event_header_tag gf_perf_event_header_t;

typedef struct gf_perf_event_gl_draw_enter_tag gf_perf_event_gl_draw_enter_t;
typedef struct gf_perf_event_gl_draw_exit_tag gf_perf_event_gl_draw_exit_t;
typedef struct gf_perf_event_cm_flush_enter_tag gf_perf_event_cm_flush_enter_t;
typedef struct gf_perf_event_cm_flush_exit_tag gf_perf_event_cm_flush_exit_t;
typedef struct gf_perf_event_swap_buffer_enter_tag gf_perf_event_swap_buffer_enter_t;
typedef struct gf_perf_event_swap_buffer_exit_tag gf_perf_event_swap_buffer_exit_t;
typedef struct gf_perf_event_present_enter_tag gf_perf_event_present_enter_t;
typedef struct gf_perf_event_present_exit_tag gf_perf_event_present_exit_t;
typedef struct gf_perf_event_dma_buffer_queued_tag gf_perf_event_dma_buffer_queued_t;
typedef struct gf_perf_event_dma_buffer_submitted_tag gf_perf_event_dma_buffer_submitted_t;
typedef struct gf_perf_event_dma_buffer_completed_tag gf_perf_event_dma_buffer_completed_t;
typedef struct gf_perf_event_lost_event_tag gf_perf_event_lost_event_t;
typedef struct gf_perf_event_vsync_tag  gf_perf_event_vsync_t;
typedef struct gf_perf_event_ps_flip_tag gf_perf_event_ps_flip_t;
typedef struct gf_perf_event_overlay_flip_tag gf_perf_event_overlay_flip_t;

typedef struct gf_perf_event_lock_enter_tag gf_perf_event_lock_enter_t;
typedef struct gf_perf_event_lock_exit_tag gf_perf_event_lock_exit_t;
typedef struct gf_perf_event_unlock_enter_tag gf_perf_event_unlock_enter_t;
typedef struct gf_perf_event_unlock_exit_tag gf_perf_event_unlock_exit_t;

typedef struct gf_perf_event_enqueue_native_kernel_enter_tag gf_perf_event_enqueue_native_kernel_enter_t;
typedef struct gf_perf_event_enqueue_native_kernel_exit_tag gf_perf_event_enqueue_native_kernel_exit_t;
typedef struct gf_perf_event_enqueue_task_enter_tag gf_perf_event_enqueue_task_enter_t;
typedef struct gf_perf_event_enqueue_task_exit_tag gf_perf_event_enqueue_task_exit_t;
typedef struct gf_perf_event_enqueue_ndr_kernel_enter_tag gf_perf_event_enqueue_ndr_kernel_enter_t;
typedef struct gf_perf_event_enqueue_ndr_kernel_exit_tag gf_perf_event_enqueue_ndr_kernel_exit_t;

typedef struct gf_perf_event_sync_event_tag gf_perf_event_sync_event_t;
typedef struct gf_perf_event_wait_start_tag gf_perf_event_wait_start_t;
typedef struct gf_perf_event_wait_finish_tag gf_perf_event_wait_finish_t;
typedef struct gf_perf_event_wait_on_server_finish_tag gf_perf_event_wait_on_server_finish_t;

typedef struct gf_perf_event_miu_counter_dump_tag gf_perf_event_miu_counter_dump_t;

typedef struct gf_perf_event_mm_lock_enter_tag gf_perf_event_mm_lock_enter_t;
typedef struct gf_perf_event_mm_lock_exit_tag gf_perf_event_mm_lock_exit_t;
typedef struct gf_perf_event_mm_unlock_enter_tag gf_perf_event_mm_unlock_enter_t;
typedef struct gf_perf_event_mm_unlock_exit_tag gf_perf_event_mm_unlock_exit_t;
typedef struct gf_perf_event_mm_alloc_enter_tag gf_perf_event_mm_alloc_enter_t;
typedef struct gf_perf_event_mm_alloc_exit_tag gf_perf_event_mm_alloc_exit_t;
typedef struct gf_perf_event_mm_free_enter_tag gf_perf_event_mm_free_enter_t;
typedef struct gf_perf_event_mm_free_exit_tag gf_perf_event_mm_free_exit_t;




typedef union gf_perf_event_tag gf_perf_event_t;

// perf event type
#define GF_PERF_EVENT_GL_DRAW_ENTER                0x1000
#define GF_PERF_EVENT_GL_DRAW_EXIT                 0x1001
#define GF_PERF_EVENT_CM_FLUSH_ENTER               0x1002
#define GF_PERF_EVENT_CM_FLUSH_EXIT                0x1003
#define GF_PERF_EVENT_SWAP_BUFFER_ENTER            0x1004
#define GF_PERF_EVENT_SWAP_BUFFER_EXIT             0x1005
#define GF_PERF_EVENT_PRESENT_ENTER                0x1006
#define GF_PERF_EVENT_PRESENT_EXIT                 0x1007
#define GF_PERF_EVENT_DMA_BUFFER_QUEUED            0x1008
#define GF_PERF_EVENT_DMA_BUFFER_SUBMITTED         0x1009
#define GF_PERF_EVENT_DMA_BUFFER_COMPLETED         0x100A
#define GF_PERF_EVENT_LOST_EVENT                   0x100B
#define GF_PERF_EVENT_VSYNC                        0x100C
#define GF_PERF_EVENT_PS_FLIP                      0x100D
#define GF_PERF_EVENT_OVERLAY_FLIP                 0x100E

#define GF_PERF_EVENT_CL_ENQUEUE_NATIVE_KERNEL_ENTER 0x100F
#define GF_PERF_EVENT_CL_ENQUEUE_NATIVE_KERNEL_EXIT  0x1010
#define GF_PERF_EVENT_CL_ENQUEUE_TASK_ENTER        0x1011
#define GF_PERF_EVENT_CL_ENQUEUE_TASK_EXIT         0x1012
#define GF_PERF_EVENT_CL_ENQUEUE_NDR_KERNEL_ENTER  0x1013
#define GF_PERF_EVENT_CL_ENQUEUE_NDR_KERNEL_EXIT   0x1014

#define GF_PERF_EVENT_LOCK_ENTER                   0x1015
#define GF_PERF_EVENT_LOCK_EXIT                    0x1016
#define GF_PERF_EVENT_UNLOCK_ENTER                 0x1017
#define GF_PERF_EVENT_UNLOCK_EXIT                  0x1018
#define GF_PERF_EVENT_SYNC_EVENT                   0x1019
#define GF_PERF_EVENT_WAIT_START                   0x101A
#define GF_PERF_EVENT_WAIT_FINISH                  0x101B
#define GF_PERF_EVENT_WAIT_ON_SERVER_FINISH        0x101C

#define GF_PERF_EVENT_HWC_FLUSH_ENTER              0x101E
#define GF_PERF_EVENT_HWC_FLUSH_EXIT               0x101F

#define GF_PERF_EVENT_MIU_COUNTER                  0x1020

#define GF_PERF_EVENT_MM_LOCK_ENTER                0x1050
#define GF_PERF_EVENT_MM_LOCK_EXIT                 0x1051
#define GF_PERF_EVENT_MM_UNLOCK_ENTER              0x1052
#define GF_PERF_EVENT_MM_UNLOCK_EXIT               0x1053
#define GF_PERF_EVENT_MM_ALLOC_ENTER               0x1054
#define GF_PERF_EVENT_MM_ALLOC_EXIT                0x1055
#define GF_PERF_EVENT_MM_FREE_ENTER                0x1056
#define GF_PERF_EVENT_MM_FREE_EXIT                 0x1057


#define GF_PERF_EVENT_INVALID_PID                  0
#define GF_PERF_EVENT_INVALID_TID                  0

#define GF_PERF_EVENT_DMA_TYPE_2D                  0x1
#define GF_PERF_EVENT_DMA_TYPE_3D                  0x2
#define GF_PERF_EVENT_DMA_TYPE_3DBLT               0x3
#define GF_PERF_EVENT_DMA_TYPE_PRESENT             0x4
#define GF_PERF_EVENT_DMA_TYPE_PAGING              0x5
#define GF_PERF_EVENT_DMA_TYPE_OVERLAY             0x6

struct gf_perf_event_header_tag
{
    unsigned int size; /* total event size */
    unsigned int type; /* event type */
    unsigned int pid; /* process id that generates the event */
    unsigned int tid; /* thead id that generates the event */
    unsigned int timestamp_low; /* ticks (or nanoseconds?) when generating the event */
    unsigned int timestamp_high;
};

struct gf_perf_event_gl_draw_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int frame_num;
    unsigned int draw_num;
};

struct gf_perf_event_gl_draw_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int frame_num;
    unsigned int draw_num;
};

struct gf_perf_event_cm_flush_enter_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_cm_flush_exit_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_swap_buffer_enter_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_swap_buffer_exit_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_present_enter_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_present_exit_tag
{
    gf_perf_event_header_t header;
};

struct gf_perf_event_dma_buffer_queued_tag
{
    gf_perf_event_header_t header;
    unsigned int gpu_context;
    unsigned int dma_idx_low;
    unsigned int dma_idx_high;
    unsigned int engine_idx;
};

struct gf_perf_event_dma_buffer_submitted_tag
{
    gf_perf_event_header_t header;
    unsigned int dma_type;
    unsigned int gpu_context; /* through gpu_context, dma_idx_low, dma_idx_high we can make connection */
    unsigned int dma_idx_low; /* between dma_buffer_queued and dma_buffer_submitted */
    unsigned int dma_idx_high;
    unsigned int engine_idx;  /* through engine_idx, fence_id_low, fence_id_high we can make connection */
    unsigned int fence_id_low; /* between dma_buffer_submitted and dma_buffer_completed */
    unsigned int fence_id_high;
};

struct gf_perf_event_dma_buffer_completed_tag
{
    gf_perf_event_header_t header;
    unsigned int engine_idx;
    unsigned int fence_id_low;
    unsigned int fence_id_high;
};

struct gf_perf_event_lost_event_tag
{
    gf_perf_event_header_t header;
    unsigned int lost_event_num;
    unsigned int lost_timestamp_low;
    unsigned int lost_timestamp_high;
};

struct gf_perf_event_vsync_tag
{
    gf_perf_event_header_t header;
    unsigned int iga_idx;
    unsigned int vsync_cnt_low;
    unsigned int vsync_cnt_high;
};

struct gf_perf_event_ps_flip_tag
{
    gf_perf_event_header_t header;
    unsigned int iga_idx;
    unsigned int allocation;
};

struct gf_perf_event_overlay_flip_tag
{
    gf_perf_event_header_t header;
    unsigned int iga_idx;
    unsigned int overlay_idx;
    unsigned int allocation;
};

struct gf_perf_event_lock_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int device;
    unsigned int handle; // locked allocation handle
    unsigned int flag; // lock flags
};

struct gf_perf_event_lock_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int device;
    unsigned int handle; // locked allocation handle
    unsigned int flag; // lock flags
};

struct gf_perf_event_unlock_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int device;
    unsigned int handle; // locked allocation handle

};

struct gf_perf_event_unlock_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int device;
    unsigned int handle; // locked allocation handle
};

struct gf_perf_event_enqueue_native_kernel_enter_tag
{
    gf_perf_event_header_t header;
    int cl_event;// is cl event, always be 1
};

struct gf_perf_event_enqueue_native_kernel_exit_tag
{
    gf_perf_event_header_t header;
    int cl_event;
};

struct gf_perf_event_enqueue_task_enter_tag
{
    gf_perf_event_header_t header;
    int cl_event;
};

struct gf_perf_event_enqueue_task_exit_tag
{
    gf_perf_event_header_t header;
    int cl_event;
};

struct gf_perf_event_enqueue_ndr_kernel_enter_tag
{
    gf_perf_event_header_t header;
    int cl_event;
};

struct gf_perf_event_enqueue_ndr_kernel_exit_tag
{
    gf_perf_event_header_t header;
    int cl_event;
};

struct gf_perf_event_sync_event_tag
{
    gf_perf_event_header_t header;
    unsigned int engine_idx;
    unsigned int type; // the sync type
    unsigned int handle;
    unsigned int fence_value_high;
    unsigned int fence_value_low;
    unsigned int gpu_context;
    unsigned int ctx_task_id_high;
    unsigned int ctx_task_id_low;
};

typedef struct gf_perf_event_wait_instance_tag
{
    unsigned int handle;
    unsigned int fence_value_high;
    unsigned int fence_value_low;
    unsigned int timeout;
}gf_perf_event_wait_instance_t;

struct gf_perf_event_wait_start_tag
{
    gf_perf_event_header_t header;
    unsigned int engine_idx;
    unsigned int gpu_context;
    unsigned int task_id_high;
    unsigned int task_id_low;
    gf_perf_event_wait_instance_t instance[32];
};

struct gf_perf_event_wait_finish_tag
{
    gf_perf_event_header_t header;
    unsigned int engine_idx;
    unsigned int gpu_context;
    unsigned int status;
    unsigned int task_id_high;
    unsigned int task_id_low;
};

struct gf_perf_event_wait_on_server_finish_tag
{
    gf_perf_event_header_t header;
    unsigned int engine_idx;
    unsigned int gpu_context;
    unsigned int task_id_high;
    unsigned int task_id_low;
};

struct gf_perf_event_miu_counter_dump_tag
{
    gf_perf_event_header_t header;
    unsigned int fence_id_high;
    unsigned int fence_id_low;
    unsigned int task_id_high;
    unsigned int task_id_low;
    unsigned int gpu_context;

    unsigned int counter_buffer_offset;
    unsigned int buffer_length;
};

struct gf_perf_event_mm_lock_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_lock_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_unlock_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_unlock_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_alloc_enter_tag
{
    gf_perf_event_header_t header;

};

struct gf_perf_event_mm_alloc_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_free_enter_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};

struct gf_perf_event_mm_free_exit_tag
{
    gf_perf_event_header_t header;
    unsigned int handle; //  allocation handle

};




union gf_perf_event_tag
{
    gf_perf_event_header_t header;
    gf_perf_event_gl_draw_enter_t gl_draw_enter;
    gf_perf_event_gl_draw_exit_t gl_draw_exit;
    gf_perf_event_cm_flush_enter_t cm_flush_enter;
    gf_perf_event_cm_flush_exit_t cm_flush_exit;
    gf_perf_event_swap_buffer_enter_t swap_buffer_enter;
    gf_perf_event_swap_buffer_exit_t swap_buffer_exit;
    gf_perf_event_present_exit_t present_enter;
    gf_perf_event_present_exit_t present_exit;
    gf_perf_event_dma_buffer_queued_t dma_buffer_queued;
    gf_perf_event_dma_buffer_submitted_t dma_buffer_submitted;
    gf_perf_event_dma_buffer_completed_t dma_buffer_completed;
    gf_perf_event_lost_event_t lost_event;
    gf_perf_event_vsync_t vsync_event;
    gf_perf_event_ps_flip_t ps_flip_event;
    gf_perf_event_overlay_flip_t overlay_flip_event;

    gf_perf_event_lock_enter_t lock_enter;
    gf_perf_event_lock_exit_t lock_exit;
    gf_perf_event_unlock_enter_t unlock_enter;
    gf_perf_event_unlock_exit_t unlock_exit;

    gf_perf_event_enqueue_native_kernel_enter_t eq_native_kernel_enter;
    gf_perf_event_enqueue_native_kernel_exit_t eq_native_kernel_exit;
    gf_perf_event_enqueue_task_enter_t eq_task_enter;
    gf_perf_event_enqueue_task_exit_t eq_task_exit;
    gf_perf_event_enqueue_ndr_kernel_enter_t eq_ndr_kernel_enter;
    gf_perf_event_enqueue_ndr_kernel_exit_t eq_ndr_kernel_exit;

    gf_perf_event_sync_event_t sync_event;
    gf_perf_event_wait_start_t wait_start;
    gf_perf_event_wait_finish_t wait_finish;
    gf_perf_event_wait_on_server_finish_t wait_on_server_finish;
    gf_perf_event_miu_counter_dump_t miu_counter;

    gf_perf_event_mm_lock_enter_t   mm_lock_enter;
    gf_perf_event_mm_lock_exit_t    mm_lock_exit;
    gf_perf_event_mm_unlock_enter_t mm_unlock_enter;
    gf_perf_event_mm_unlock_exit_t  mm_unlock_exit;
    gf_perf_event_mm_alloc_enter_t  mm_alloc_enter;
    gf_perf_event_mm_alloc_exit_t   mm_alloc_exit;
    gf_perf_event_mm_free_enter_t   mm_free_enter;
    gf_perf_event_mm_free_exit_t    mm_free_exit;

};

typedef struct gf_perf_status_tag
{
    int started; /* out */
    int miu_started; /* out */
} gf_perf_status_t;

typedef struct
{
    unsigned short  Vendor_ID;
    unsigned char   Device_ID;
    unsigned char   Revision_ID;
}gf_chip_id_t;

#define GF_HDCP_ENABLED            0x0
#define GF_HDCP_FAILED             0x1
#define GF_HDCP_NO_SUPPORT_DEVICE  0x2

typedef struct
{
    unsigned int    enable;                  /* in, 1 enable, 0 disable */
    unsigned int    output_type;             /* in, if 0, all HDMIs will be enable/disable */
    unsigned int    result;                  /* out, see GF_HDCP_XXX */
}gf_hdcp_op_t;

typedef struct
{
    unsigned int  hAllocation;
    unsigned int  engine_mask;
    unsigned int  state;
}gf_get_allocation_state_t;

typedef enum
{
    GF_HW_NONE = 0x00,
    GF_HW_IGA  = 0x01,
}gf_hw_block;

#define STREAM_MASK_UPDATE  0x01
#define STREAM_MASK_FLIP    0x02

#define GF_NAME "gf"

#define     UT_OUTPUT_TYPE_NONE      0x00
#define     UT_OUTPUT_TYPE_CRT       0x01
#define     UT_OUTPUT_TYPE_TV        0x02
#define     UT_OUTPUT_TYPE_HDTV      0x04
#define     UT_OUTPUT_TYPE_PANEL     0x08
#define     UT_OUTPUT_TYPE_DVI       0x10
#define     UT_OUTPUT_TYPE_HDMI      0x20
#define     UT_OUTPUT_TYPE_DP        0x40
#define     UT_OUTPUT_TYPE_MHL       0x80

static inline gf_ptr64_t ptr_to_ptr64(void *ptr)
{
    return (gf_ptr64_t)(unsigned long)ptr;
}

static inline void *ptr64_to_ptr(gf_ptr64_t ptr64)
{
    return (void*)(unsigned long)ptr64;
}

typedef struct {
    unsigned int         pid;
    unsigned int         enable;
    unsigned int         dwDecodeLevel;
    unsigned int         dwDecodeType;
    unsigned int         dwFrameType;
    int                  DecodeWidth;
    int                  DecodeHeight;
    unsigned int         DecodeRTNum;
    unsigned int         SliceNum;

    unsigned int         TotalFrameNum;
    unsigned int         TotalIFrameNum;
    unsigned int         TotalPFrameNum;
    unsigned int         TotalBFrameNum;
    unsigned int         TotalBltFrameNum;
    unsigned int         TotalBitstreamSize;
}gf_vcp_info;

typedef struct
{
    unsigned short  VendorID;                   // (ro)
    unsigned short  DeviceID;                   // (ro)
    unsigned short  Command;                    // Device control
    unsigned short  Status;
    unsigned char   RevisionID;                 // (ro)
    unsigned char   ProgIf;                     // (ro)
    unsigned char   SubClass;                   // (ro)
    unsigned char   BaseClass;                  // (ro)
    unsigned char   CacheLineSize;              // (ro+)
    unsigned char   LatencyTimer;               // (ro+)
    unsigned char   HeaderType;                 // (ro)
    unsigned char   BIST;                       // Built in self test
    unsigned int    ulBaseAddresses[6]; // 6 == PCI_TYPE0_ADDRESSES
    unsigned int    CardBus;
    unsigned short  SubsystemVendorID;
    unsigned short  SubSystemID;
    unsigned int    PCIConfig_Unused[4];
    unsigned int    AGPCapability;
    unsigned int    AGPStatus;
    unsigned int    AGPCommand;
    unsigned int    BusNum;
    unsigned int    SlotNum;

    // for PCI Express support
    unsigned int    LinkCaps;
    unsigned short  LinkStatus;

    int             bSwizzled;
    unsigned int    memoryType;      // 0: 128bits bus; 1: 64bits bus
    unsigned char   pageSelect;      // For memory (2K, 4K, 8K page sizes)
    unsigned char   bankSelect;      // Which address bits to use for bank select
    unsigned char   bankSelectShift; // Which address bits to use for bank select
    unsigned char   bankSelectMask;  // BitMask for bank select bits
    unsigned long long   memorySize;
} gf_pci_config_t;

typedef struct {
    unsigned int VidmemSize;
    unsigned int cjGARTTable;
    unsigned int GARTTableSize;
    unsigned int cjVidmemPageTable;
    unsigned int VidmemPageTableSize;
    unsigned int cjPageKeyBuffer;
    unsigned int PageKeyBufferSize;
    unsigned int cjFBRingBuffer;
    unsigned int FBRingBufferSize;
    int	bNullHW;
    int	bNullBridge;
    unsigned long UserCModelVidMemBase;
    unsigned long pfnCModelFlushEngine;
    unsigned long pfnGetCModelInfo;
    unsigned long pfnGetRegistryFromCModelIni;
} gf_model2_cil_info_t;

typedef struct
{
    gf_pci_config_t            bus_config;
    unsigned int               chipslicemask;
    unsigned int               gpucount;
    unsigned int               osversion;
    int                        bVideoOnly;
    int                        bCTEDumpEnable;
    gf_model2_cil_info_t       model2cilinfo;
    int                        segmentMapTable[5];
} gf_adapter_info_t;

#endif
